package de.hft.stuttgart.citydoctor2.checks.geometry;

import static org.junit.Assert.assertTrue;

import java.util.List;

import org.junit.Before;
import org.junit.Test;

import de.hft.stuttgart.citydoctor2.check.CheckError;
import de.hft.stuttgart.citydoctor2.check.CheckResult;
import de.hft.stuttgart.citydoctor2.check.ErrorId;
import de.hft.stuttgart.citydoctor2.check.ResultStatus;
import de.hft.stuttgart.citydoctor2.check.error.InteriorFaceError;
import de.hft.stuttgart.citydoctor2.datastructure.Geometry;
import de.hft.stuttgart.citydoctor2.datastructure.Polygon;
import de.hft.stuttgart.citydoctor2.datastructure.Vertex;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertNotNull;
import static de.hft.stuttgart.citydoctor2.checks.util.GeometryTestUtils.createGoodGeometry;
import static de.hft.stuttgart.citydoctor2.checks.util.GeometryTestUtils.createVertex;
import static de.hft.stuttgart.citydoctor2.checks.util.GeometryTestUtils.addPolygon;

public class InteriorFacesCheckTest {

	Geometry geom;

	@Before
	public void setup() {
		geom = createGoodGeometry();
	}

	@Test
	public void testGoodGeometry() {
		InteriorFacesCheck check = new InteriorFacesCheck();
		check.check(geom);
		CheckResult cr = geom.getCheckResult(check);
		assertNotNull("Check result must not be null", cr);
		assertEquals("Good geometry must return OK check result", ResultStatus.OK, cr.getResultStatus());
	}
	
	@Test
	public void testSimpleGeometryWithInteriorFace() {
		Vertex v0 = createVertex(5,0,0,geom);
		Vertex v1 = createVertex(5,0,10,geom);
		Vertex v2 = createVertex(5,10,10,geom);
		Vertex v3 = createVertex(5,10,0,geom);
		Polygon insertedInteriorFace = addPolygon(geom, v0, v1, v2, v3);
		geom.updateEdgesAndVertices();
		
		InteriorFacesCheck check = new InteriorFacesCheck();
		check.check(geom);
		CheckResult cr = geom.getCheckResult(check);
		assertNotNull(cr);
		assertEquals("Check should have detected an error", ResultStatus.ERROR, cr.getResultStatus());
		assertEquals("ID of found Error should be GE_S_INTERIOR_FACE",ErrorId.GE_S_INTERIOR_FACE,cr.getError().getErrorId());
		
		CheckError foundError = cr.getError();
		assertTrue("Check should have returned a InteriorFaceError",foundError instanceof InteriorFaceError);
		
		List<Polygon> interiorFaces = ((InteriorFaceError) foundError).getPolygons();
		assertNotNull(interiorFaces);
		assertTrue("InteriorFaces should have a single entry", interiorFaces.size() == 1);
		
		Polygon foundInteriorFace = interiorFaces.getFirst();
		assertEquals("Found interior face should be the inserted polygon", insertedInteriorFace, foundInteriorFace);
	}
	

}
